static const char* const rcsid = "$Id$";
/*                               -*- Mode: C -*- 
 * oprs-mrPost.c -- 
 * 
 * Copyright (C) 1994-2012 LAAS/CNRS
 * 
 * Author          : Francois Felix Ingrand
 * Created On      : Tue Apr 26 11:37:29 1994
 * Last Modified By: Francois Felix Ingrand
 * Last Modified On: Tue Jul 25 12:11:21 2000
 * Update Count    : 114
 * Status          : OK
 * 
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>

#include <hashPack.h>

#include "macro.h"

#include "opaque-pub.h"
#include "macro-pub.h"
#undef FREE

#include "oprs-type-pub.h"
#include "pu-mk-term_f.h"
#include "pu-parse-tl_f.h"

#include "oprs-type_f-pub.h"
#include "lisp-list_f-pub.h"


#include "genom3/c/client.h"

#include "genom-oprs.h"
#include "genom-oprs_f.h"
#include "genom-oprs-com.h"
#include "genom-oprs-com_f.h"
#include "genom-oprs-post_f.h"

void end_oprs_post ( void );
void init_oprs_post ( void );

typedef struct {
     char *name;
     const struct genom_port_info *genom_port_info;
     Decode_Func_Proto decode_func;
} Poster;


#define POSTER_HASH_SIZE 16		/* has to be a power of two */
static int poster_hashtable_size = POSTER_HASH_SIZE;
static int poster_hashtable_mod = POSTER_HASH_SIZE -1;

static int poster_hash_a_string(PString name )	
{
     return (hash_a_string(name) & poster_hashtable_mod);
}

static PBoolean poster_match(PString f1, Poster *poster)
{
     return ( f1 == poster->name) ;
}

HASH make_poster_hashtable() {
     return make_hashtable(poster_hashtable_size,
			   poster_hash_a_string,
			   poster_match);
}

void end_oprs_post()
{
}

void init_oprs_post()
{
}

void declare_poster(char *name, Mod *mod,
		    const struct genom_port_info *genom_port_info, 
		    Decode_Func_Proto decode_func)
{
     Poster *poster = MAKE_OBJECT(Poster);
     
     poster->name = declare_atom(name);	/* Declare the poster as a symbol */
     poster->decode_func = decode_func;
     poster->genom_port_info = genom_port_info;

     if ((get_from_hashtable(mod->poster_hash, poster->name)) != NULL)
	  fprintf(stderr, "genom-oprs: The poster \"%s\" has already been declared.\n", poster->name); 
     
     add_to_hashtable(mod->poster_hash, poster, poster->name); 

     return;
}   


Term *read_poster_internal(char *mod_name, char *poster_name, char *name)
{
     Poster *poster;
     Term *res;
     Mod *mod;

     if ((mod = (Mod *)get_from_hashtable(mod_hash,
					  mod_name)) == NULL) {
	  fprintf(stderr, "genom-oprs:read_poster: could not find module: %s.\n", mod_name); 
	  return build_nil();
     } else if ((poster = (Poster *)get_from_hashtable(mod->poster_hash,
						poster_name)) == NULL) {
	  fprintf(stderr, "genom-oprs:read_poster: could not find poster: %s.\n", poster_name); 
	  return build_nil();
     } else if (!mod->initialized && !init_oprs_mod_com(mod)) {
	  fprintf(stderr, "genom-oprs:read_poster: module %s not initialized.\n", mod_name);
	  return build_nil();
     } else {
	  void *data;
	  const struct genom_port_info *gpi;
	  
	  gpi = poster->genom_port_info;
	  
	  data = malloc(gpi->data_size);
	  gpi->init_data(data);
	  if (gpi->simple) {
	       if (name) 
		    fprintf(stderr, "genom-oprs:read_poster: %s seems to be a regular poster, but you are passing another name argument: %s.\n", poster_name, name); 
	       gpi->read.simple(mod->genom_client, data);
	  } else {
	       if (!name) {
		    fprintf(stderr, "genom-oprs:read_poster: %s seems to be a  poster, but you forgot the extra argument.\n", poster_name);
		    return build_nil();
	       } else {
		    gpi->read.multiple(mod->genom_client, name, data);
	       }
	  }
	  res = (*(poster->decode_func))(data);
	  gpi->fini_data(data);
	  free(data);
	  return res;
     }
}
     
Term *read_poster_name(TermList terms)
/* This is the action which read the poster. */
{
     char *poster_name;
     char *name;
     char *mod_name;

     if (! (PUGetOprsParameters(terms, 3, ATOM, &mod_name, ATOM, &poster_name, ATOM, &name))) {
	  fprintf(stderr, "genom-oprs:read_poster_name: unable to parse read_poster_name arguments.\n"); 
	  return  build_nil();
     }
     
     return read_poster_internal(mod_name, poster_name, name);
}

Term *read_poster(TermList terms)
/* This is the action which read the poster. */
{
     char *poster_name;
     char *mod_name;

     if (! (PUGetOprsParameters(terms, 2, ATOM, &mod_name, ATOM, &poster_name))) {
	  fprintf(stderr, "genom-oprs:read_poster: unable to parse read_poster argument.\n"); 
	  return  build_nil();
     }
     return read_poster_internal(mod_name, poster_name, NULL);
}

